package com.macro.mall.tiny.modules.business.controller;

import com.macro.mall.tiny.common.api.CommonResult;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.dto.FileUploadParam;
import com.macro.mall.tiny.modules.business.service.FileService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.Principal;

/**
 * 大文件上传
 */
@Slf4j
@RestController
@RequestMapping("/file")
@CrossOrigin
@Api(tags = "FileController", description = "提供文件上传相关API")
public class FileController {
    @Autowired
    private FileService fileService;


    /**
     * @param fileUploadParam
     * @return void
     * @describe 文件上传
     * @author gjs
     * @date 2021/3/25
     * @since J7.23.0
     */
    @ApiOperation("文件上传接口")
    @PostMapping("/upload")
    public void upload(FileUploadParam fileUploadParam, Principal principal) throws IOException {
        String name = fileUploadParam.getName();
        String lineName = fileUploadParam.getLineName();
        Long provinceId = fileUploadParam.getProvince();
        Long size = fileUploadParam.getSize();
        Integer chunks = fileUploadParam.getChunks();
        Integer chunk = fileUploadParam.getChunk();
        MultipartFile file = fileUploadParam.getFile();

        if (chunks != null && chunks != 0) {
            fileService.uploadWithBlock(name, lineName, provinceId, size, chunks, chunk, file, principal);
        } else {
            fileService.upload(file);
        }
    }

    @ApiOperation("文件下载接口")
    @GetMapping("/download")
    public Object downloadFile(@RequestParam String fileName, final HttpServletResponse response) {
        OutputStream os = null;
        InputStream is = null;
        try {
            // 取得输出流
            os = response.getOutputStream();
            // 清空输出流
            response.reset();
            response.setContentType("application/x-download;charset=UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            //读取流
            File f = new File(UploadConfig.path + fileName);
            is = new FileInputStream(f);
            //复制
            IOUtils.copy(is, response.getOutputStream());
            response.getOutputStream().flush();
        } catch (IOException e) {
            return CommonResult.failed("下载附件失败,error:" + e.getMessage());
        }
        //文件的关闭放在finally中
        finally {
            try {
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                log.error("关闭文件输入流失败：", e);
            }
            try {
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                log.error("关闭文件输出流失败：", e);
            }
        }
        return null;
    }

    /**
     * 下载模板
     */
    @ApiOperation("下载杆塔信息录入模板")
    @GetMapping("/downloadTemplate")
    public void downloadTemplate(HttpServletResponse response) throws Exception {
        ClassPathResource classPathResource = new ClassPathResource("excel-template/杆塔信息录入.xlsx");
        InputStream inputStream = classPathResource.getInputStream();
        Workbook workbook = new XSSFWorkbook(inputStream);
        response.setContentType("application/vnd.ms-excel");
        response.setHeader("content-Disposition", "attachment;filename=" + URLEncoder.encode("杆塔信息录入.xlsx", "utf-8"));
        response.setHeader("Access-Control-Expose-Headers", "content-Disposition");
        OutputStream outputStream = response.getOutputStream();
        workbook.write(outputStream);
        outputStream.flush();
        outputStream.close();
    }
}
