package com.macro.mall.tiny.modules.business.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.macro.mall.tiny.common.exception.ApiException;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.CommonUtils;
import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.enume.DirectoryEnum;
import com.macro.mall.tiny.modules.business.model.*;
import com.macro.mall.tiny.modules.business.service.BLineService;
import com.macro.mall.tiny.modules.business.service.BLineUploadLogService;
import com.macro.mall.tiny.modules.business.service.BTowerService;
import com.macro.mall.tiny.modules.business.service.FileService;
import com.macro.mall.tiny.modules.ums.model.UmsAdmin;
import com.macro.mall.tiny.modules.ums.model.UmsRole;
import com.macro.mall.tiny.modules.ums.service.UmsAdminService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.security.Principal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import static com.macro.mall.tiny.common.util.UploadUtils.*;


/**
 * 文件上传服务
 */
@Service
@Slf4j
@EnableAsync
public class FileServiceImpl implements FileService {

    @Autowired
    private AsyncHandler asyncHandler;

    /**
     * 上传文件
     *
     * @param file
     */
    public void upload(MultipartFile file) throws IOException {
        String path = UploadConfig.path + file.getOriginalFilename();
        MyFileUtils.write(path, file.getInputStream());
    }

    /**
     * 分块上传文件
     *
     * @param size
     * @param chunks
     * @param chunk
     * @param file
     * @throws IOException
     */
    public void uploadWithBlock(String name,
                                String lineName,
                                Long provinceId,
                                Long size,
                                Integer chunks,
                                Integer chunk,
                                MultipartFile file,
                                Principal principal) throws IOException {
        putChunkMap(name, chunks);
        String target = UploadConfig.tempPath + name;
        MyFileUtils.writeWithBlock(target, size, file.getInputStream(), file.getSize(), chunks, chunk);
        addChunk(name, chunk);
        if (isUploaded(name)) {
            removeKey(name);
            asyncHandler.handleDocument(provinceId, target, lineName, principal);
        }
    }

}
